﻿// @Copyright (c) 2019, citicsf Co. Ltd. All Rights Reserved
// @Author:   citicsf
// @Date:     2019/08/20
// @Brief:

#ifndef __REMD_REMDAPISTRUCT_H__
#define __REMD_REMDAPISTRUCT_H__

#include "stdint.h"

// 消息类型定义
enum REMD_MSG_TYPE {
    REMD_MSG_NONE = -1,

    REMD_SYS_UDP_REALTIME_TIMEOUT = 0,      // UDP实时数据通道超时
    REMD_SYS_TCP_RETRANSMIT_CONNECTED,      // TCP重传数据通道连接成功
    REMD_SYS_TCP_RETRANSMIT_DISCONNECTED,   // TCP重传数据通道断开
    REMD_SYS_SERVER_LOGIN,                  // 连接服务器
    REMD_SYS_CODE_TABLE_UPDATED,            // 代码表更新
    REMD_SYS_DATA_REBUILD,                  // 数据重建消息

    REMD_DATA_INDEX = 100,          // 指数数据
    REMD_DATA_SECURITY,             // 证券行情快照
    REMD_DATA_FUTURE,               // 期货/期权行情快照
    REMD_DATA_TRADE_DETAIL,         // 逐笔成交
    REMD_DATA_ORDER_DETAIL,         // 逐笔委托
    REMD_DATA_ORDER_QUEUE,          // 委托队列
    REMD_DATA_SH_ORDER_DETAIL,         // 2021.04.25 上海逐笔委托
};

typedef uint64_t SeqType;

struct REMD_MSG {
    // 数据类型 见enum REMD_MSG_TYPE
    int     nDataType;
    // 数据长度
    int     nDataLen;
    // 服务器时间戳（精确到毫秒HHMMSSmmm）
    int     nServerTime;
    // 连接ID，在Open配置里面设定的值
    int     nConnectId;
    // 序列号，全局有序，结构化数据和原始数据独立编号
    // 如果仅订阅部分市场或代码，序列号会不连续
    // 对于交易所原始数据，上海和深圳独立编号，每日从0开始
    SeqType nSequence;
    //数据指针
    void* pData;
};

// 网络连接结果，消息类型为 REMD_SYS_TCP_RETRANSMIT_CONNECTED 和 REMD_SYS_UDP_REALTIME_TIMEOUT
struct REMD_ConnectResult {
    // 连接的服务器地址和端口号
    char        szAddress[32];
    uint16_t    nPort;
    uint16_t    nConnectResult;     // 为0则表示连接失败，非0则表示连接成功
};

// 网络断开，消息类型为 REMD_SYS_TCP_RETRANSMIT_DISCONNECTED
struct REMD_Disconnect {
    // 连接的服务器地址和端口号
    char        szAddress[32];
    uint16_t    nPort;
};

// 登录结果，消息类型为REMD_SYS_SERVER_LOGIN
struct REMD_LoginResult {
    char        szAddress[32];
    uint16_t    nPort;
    uint16_t nLoginResult;       // 为0则表示登陆验证成功，非0则表示验证失败
    char     szInfo[128];        // 错误描述
    uint16_t markets;           // 市场个数
    char     szMarkets[6][8];    // 市场代码 SZ, SH
};

// 代码表更新，消息类型为REMD_SYS_CODE_TABLE_UPDATED
struct REMD_CodeTableResult {
    char        szInfo[128];         //代码表结果文本
    int32_t     markets;           //市场个数
    char        szMarket[8][8];    //市场代码
    int32_t     nCodeCount[8];     //代码表项数
    int32_t     nCodeDate[8];      //代码表日期
};

// 数据重建消息，消息类型为REMD_SYS_DATA_REBUILD
struct REMD_RebuildResult {
    SeqType Start;      // 起始序列号
    SeqType End;        // 结束序列号
    uint32_t Status;    // 状态，0表示重传完成，1表示请求错误，2表示前一请求尚未完成，3表示重传过程中发生错误
};

//
typedef void* REMDHandle;

// 数据消息回调函数
typedef void(*REMD_DataHandler)(REMDHandle, const REMD_MSG*);

// 系统消息回调函数
typedef void(*REMD_SystemHandler)(REMDHandle, const REMD_MSG*);

// 数据订阅类型定义，订阅多个类型时使用 '|'
// 如同时订阅逐笔成交和逐笔委托： REMD_TRADE_DETAIL | REMD_ORDER_DETAIL
// 快照行情为默认订阅，故为0
enum REMD_SUBSCRIBED_DATA_TYPE {
    REMD_DATA_TYPE_SNAPSHOT = 0x00,         // 快照行情
    REMD_DATA_TYPE_TRADE_DETAIL = 0x02,         // 逐笔成交
    REMD_DATA_TYPE_ORDER_DETAIL = 0x04,         // 逐笔委托
    REMD_DATA_TYPE_ORDER_QUEUE = 0x08          // 委托队列
};

// 订阅更新类型，{全市场订阅，设置为列表证券，增加列表证券，删除列表证券}
// 在函数REMD_UpdateSubscription中使用
enum REMD_SUBSCRIPTION_UPDATE {
    REMD_ALL, REMD_SET, REMD_ADD, REMD_DEL
};

// 连接配置
struct REMD_OpenSetting {
    // 连接ID，返回的消息首部中带回
    int32_t ConnectID;

    // 服务端IP地址

    // 实时数据地址和端口(UDP组播地址)
    char     MulitcastIP[32];
    uint16_t MulitcastPort;
    char     MulitcastInterface[32];

    // 重传数据地址端口(TCP地址)
    char     RetransmitAddr[32];
    uint16_t RetransmitPort;

    // 用户名和密码
    char UserName[64];
    char Password[64];

    // 数据回调 & 系统状态回调
    // 接收到数据之后应当立即处理，避免数据积压
    REMD_DataHandler   DataHandler;
    REMD_SystemHandler SystemHandler;

    /* 订阅信息，大小写不敏感 */
    // 订阅的交易所列表，以;分隔，为空表示所有，目前支持SH,SZ
    const char* SubscribedMarkets;
    // 订阅的证券代码，以;分隔，为空表示所有，例如"600000.SH;000001.SZ"
    const char* SubscribedCodes;
    // 订阅的数据类型，REMD_SUBSCRIBED_DATA_TYPE
    int32_t SubscribedDataType;

    /* 将API绑定到指定CPU上，-1表示不指定，使用memset初始化时需特别注意 */
    int BindCPU;
};

// 原始数据流连接配置
struct REMD_RawOpenSetting {
    // 连接ID，返回的消息首部中带回
    int32_t ConnectID;

    // 服务端IP地址

    // 实时数据地址和端口(UDP组播地址)
    char     MulitcastIP[32];
    uint16_t MulitcastPort;
    char     MulitcastInterface[32];

    // 重传数据地址端口(TCP地址)
    char     RetransmitAddr[32];
    uint16_t RetransmitPort;

    // 用户名和密码
    char UserName[64];
    char Password[64];

    // 数据回调 & 系统状态回调
    // 接收到数据之后应当立即处理，避免数据积压
    REMD_DataHandler   DataHandler;
    REMD_SystemHandler SystemHandler;

    /* 订阅信息，大小写不敏感 */
    // 订阅的交易所列表，以;分隔，为空表示无（！！注：和REMD_OpenSetting不同），目前支持SH,SZ
    const char* SubscribedMarkets;

    /* 将API绑定到指定CPU上，-1表示不指定，使用memset初始化时需特别注意 */
    int BindCPU;
};

///证券原始代码
typedef char REMD_Code[16];

///系统内部代码，格式为：原始代码.市场(SH/SZ)
typedef char REMD_RemdCode[32];

///交易所ID
typedef char REMD_Market;

#define REMD_MARKET_NO  '0'
#define REMD_MARKET_SH  '1'
#define REMD_MARKET_SZ  '2'

///订单类型
typedef char REMD_OrderType;
// 市价
#define REMD_ORDER_TYPE_MARKET  '1'
// 限价
#define REMD_ORDER_TYPE_LIMIT   '2'
// 本方最优
#define REMD_ORDER_TYPE_AT_BEST  'U'

///买卖方向
typedef char REMD_OrderSide;

#define REMD_ORDER_SIDE_BID 'B'
#define REMD_ORDER_SIDE_ASK 'S'
#define REMD_ORDER_SIDE_G 'G'
#define REMD_ORDER_SIDE_F 'F'

///成交类型,{成交、撤销}
typedef char REMD_ExecType;

#define REMD_EXEC_TRADE 'T'
#define REMD_EXEC_CANCEL 'C'


///BS标记，{外盘、内盘、未知}
typedef char REMD_BSFlag;

#define REMD_BSFLAG_B  'B'
#define REMD_BSFLAG_S  'S'
#define REMD_BSFLAG_N  ' '

///上交所逐笔委托类型
typedef char REMD_SHOrderType;
#define REMD_SH_ORDER_ADD 'A'
#define REMD_SH_ORDER_DEL 'D'

///上交所逐笔数据状态类型(int类型)
//1 = 重复数据（无需处理和展示， 只要检查序号是否连续）
//2 = 未获授权
typedef char REMD_SHDataStatus;
// 正常
#define REMD_SH_DATASTATUS_NORMAL 0
// 重复数据（无需处理和展示，只要检查序号是否连续）
#define REMD_SH_DATASTATUS_DUP    1
// 未获授权
#define REMD_SH_DATASTATUS_UNAUTH 2

///交易状态
typedef char REMD_TradingStatus;

// 市场交易状态定义
enum REMD_MARKET_STATUS {
    Y = 'Y',    //'Y' 新产品.(New Product)
    P = 'P',    //'P' 休市，例如，午餐休市。无撮合市场内部信息披露。(Market Break)
    B = 'B',    //'B' 停牌.(Suspension)
    C = 'C',    //'C' 收市.(Market Closed)
    Z = 'Z',    //'Z' 产品待删除.(Deleted Product)
    V = 'V',    //'V' 结束交易.(End of Trade)
    W = 'W',    //'W' 暂停，除了自有订单和交易的查询之外，任何交易活动都被禁止。(Pause, Trades/Orders Query Only)
    X = 'X',    //'X' 停牌,( 'X'和'W'的区别在于 'X'时可以撤单).(Pause, Allow Trades/Orders Query and Cancelling Order)
    N = 'N',    //'N' 盘中集合竞价订单簿平衡.(Call auction Order Book Balance)
    L = 'L',    //'L' 盘中集合竞价PreOBB.(Call auction PreOBB)
    I = 'I',    //'I' 开市集合竞价.(Market-Open Call auction)
    J = 'J',    //'J' 收盘集合竞价.(Market-Close Call auction)
    U = 'U',    //'U' 盘后处理.(After-Market Process)
    F = 'F',    //'F' 盘前处理.(Pre-Market Process)
    E = 'E',    //'E' 启动.(Start)
    O = 'O',    //'O' 连续撮合.(Continous Matching/Trading)
    Q = 'Q',    //'Q' 连续交易和集合竞价的波动性中断.(Temporarily Circuit Breaker)
    NONE = 0
};

///基本代码表
struct REMD_CODEINFO {
    REMD_Code     code;           ///证券代码
    REMD_Market   market;          ///交易所ID
    REMD_RemdCode szRemdCode;       ///系统内部代码
    char          szCNName[32];     ///证券中文名
    int           type;            ///证券类型详见文件尾
};

///期权代码表
struct REMD_OPTION_CODEINFO {
    ///基础信息
    REMD_CODEINFO basicCode;
    ///期权合约代码
    char          szContractID[32];
    ///标的证券代码
    char          szUnderlyingSecurityID[32];
    ///认购认沽：认购，C；认沽，P
    char          callOrPut;
    ///期权行权日，YYYYMMDD
    int           exerciseDate;
    ///标的证券类型：A股，0；ETF，1
    char          underlyingType;
    ///欧式美式：欧式期权，E；美式期权，A
    char          optionType;
    ///涨跌幅限制类型：‘有涨跌幅限制类型，N, 无涨跌幅限制类型，R
    char          priceLimitType;
    ///合约单位，经过除权除息调整后的合约单位，一定是整数
    int           contractMultiplierUnit;
    ///期权行权价，经过除权除息调整后的期权行权价，右对齐，精确到厘
    int64_t       exercisePrice;
    ///期权首个交易日，YYYYMMDD
    int           startDate;
    ///期权最后交易日/行权日，YYYYMMDD
    int           endDate;
    ///期权到期日，YYYYMMDD
    int           expireDate;
};

///行情快照
struct REMD_SecuritySnapShot {
    REMD_Code   code;                   ///证券代码
    REMD_Market market;                  ///交易所ID
    int32_t     actionDay;               ///业务发生日(自然日)
    int32_t     tradingDay;              ///交易日
    int32_t     time;                    ///时间(HHMMSSmmm)
    int32_t     status;                  ///当前证券状态，REMD_MARKET_STATUS
    int64_t     preClose;                ///前收盘价
    int64_t     open;                    ///开盘价
    int64_t     high;                    ///最高价
    int64_t     low;                     ///最低价
    int64_t     last;                   ///最新价
    int64_t     askPrice[10];            ///申卖价
    int64_t     askVol[10];              ///申卖量
    int64_t     bidPrice[10];            ///申买价
    int64_t     bidVol[10];              ///申买量
    int32_t     numTrades;               ///成交笔数
    int64_t     volume;                  ///成交总量
    int64_t     turnover;                ///成交总金额
    int64_t     totalBidVol;             ///委托买入总量
    int64_t     totalAskVol;             ///委托卖出总量
    int64_t     weightedAvgBidPrice;     ///加权平均委买价格
    int64_t     weightedAvgAskPrice;     ///加权平均委卖价格
    int32_t     IOPV;                    ///IOPV净值估值
    int32_t     yieldToMaturity;         ///到期收益率
    int64_t     highLimited;             ///涨停价
    int64_t     lowLimited;              ///跌停价
    int32_t     nSyl1;                    ///市盈率1
    int32_t     nSyl2;                    ///市盈率2
    int32_t     nSD1;                     ///升跌1
    int32_t     nSD2;                     ///升跌2
    int64_t     afterPrice;              ///盘后价格(科创板)
    int64_t     afterVolume;             ///盘后量(科创板)
    int64_t     afterTurnover;           ///盘后成交金额(科创板)
    int64_t     afterMatchItems;         ///盘后成交笔数(科创板)
    int64_t     nReserve;                 ///预留位
};

///指数快照行情
struct REMD_IndexSnapShot {
    REMD_Code   code;             ///证券代码          
    REMD_Market market;          ///交易所ID          
    int32_t     actionDay;       ///业务发生日(自然日)          
    int32_t     tradingDay;      ///交易日          
    int32_t     time;            ///时间(HHMMSSmmm)          
    int64_t     preCloseIndex;   ///前收盘指数          
    int64_t     openIndex;       ///今开盘指数          
    int64_t     highIndex;       ///最高指数          
    int64_t     lowIndex;        ///最低指数          
    int64_t     lastIndex;       ///最新指数          
    int64_t     totalVolume;     ///参与计算相应指数的交易数量          
    int64_t     turnover;        ///参与计算相应指数的成交金额          
};

///委托队列
struct REMD_OrderQueue {
    REMD_Code      code;           ///证券代码            
    REMD_Market    market;        ///交易所ID            
    int32_t        actionDay;     ///业务发生日(自然日)            
    int32_t        tradingDay;    ///交易日            
    int32_t        time;          ///时间(HHMMSSmmm)            
    REMD_OrderSide side;          ///买卖方向            
    int64_t        price;         ///委托价格            
    int32_t        orders;        ///订单数量            
    int32_t        ABItems;       ///委托队列个数            
    int32_t        ABVolume[200]; ///委托明细            
};

///逐笔委托
struct REMD_OrderDetail {
    REMD_Code      code;           ///证券代码             
    REMD_Market    market;        ///交易所ID             
    int32_t        actionDay;     ///委托日期(YYMMDD)             
    int32_t        time;          ///委托时间(HHMMSSmmm)             
    uint64_t       order;         ///委托号             
    int64_t        price;         ///委托价格             
    int32_t        volume;        ///委托数量             
    REMD_OrderSide orderSide;     ///买卖方向             
    REMD_OrderType orderType;     ///订单类型             
};

///上交所逐笔委托(202105新增),与深证逐笔委托消息内容字段有所不同，故分开定义。
struct REMD_SH_OrderDetail {
    REMD_Code           code;          ///证券代码
    REMD_SHDataStatus   dataStatus;    ///上交所逐笔委托数据状态(直接转发交易所状态值，参考交易所文档)
    uint32_t            channelNo;     ///通道号
    uint32_t            orderIndex;    ///委托号
    int32_t             time;          ///委托时间(HHMMSSmmm)
    int64_t             orderNo;       ///原始订单号
    int64_t             price;         ///委托价格（*10000） 
    int64_t             balance;       ///委托数量
    REMD_BSFlag         BSFlag;        ///内外盘类型
    REMD_SHOrderType    orderType;     ///逐笔委托类型
    int64_t             bizIndex;      ///业务索引        
};

///逐笔成交
struct REMD_TradeDetail {
    REMD_Code     code;        ///证券代码
    REMD_Market   market;      ///交易所ID               
    int32_t       actionDay;   ///自然日               
    int32_t       time;        ///成交时间(HHMMSSmmm)               
    uint64_t      index;       ///成交编号               
    int64_t       price;       ///成交价格               
    int32_t       volume;      ///成交数量               
    int64_t       turnover;    ///成交金额               
    uint64_t      askOrder;    ///卖方委托序号               
    uint64_t      bidOrder;    ///买方委托序号               
    REMD_ExecType execType;    ///成交类型               
    REMD_BSFlag   BSFlag;      ///内外盘类型    
    int32_t       channelNo;   ///通道号(202105新增，上证字段，深证该字段为0)
    int64_t       bizIndex;    ///业务号(202105新增，上证为业务号，深证该字段为消息记录号"ApplSeqNum"，从1开始计数)
};

///期货行情，目前只提供期权数据
struct REMD_OptionSnapShot {
    REMD_Code   code;             ///证券代码
    REMD_Market market;          ///交易所ID       
    int32_t     actionDay;       ///业务发生日(自然日)       
    int32_t     tradingDay;      ///交易日       
    int32_t     time;            ///时间(HHMMSSmmm)       
    int32_t     status;          ///当前证券状态，REMD_MARKET_STATUS       
    int64_t     preOpenInterest; ///昨持仓       
    int64_t     preClose;        ///昨收盘价       
    int64_t     preSettlePrice;  ///昨结算       
    int64_t     open;            ///开盘价       
    int64_t     high;            ///最高价       
    int64_t     low;             ///最低价       
    int64_t     last;           ///最新价       
    int64_t     volume;          ///成交总量       
    int64_t     turnover;        ///成交总金额       
    int64_t     openInterest;    ///持仓总量       
    int64_t     close;           ///今收盘       
    int64_t     settlePrice;     ///今结算       
    int64_t     highLimited;     ///涨停价       
    int64_t     lowLimited;      ///跌停价       
    int32_t     preDelta;        ///昨虚实度       
    int32_t     currDelta;       ///今虚实度       
    int64_t     askPrice[5];     ///申卖价       
    int64_t     askVol[5];       ///申卖量       
    int64_t     bidPrice[5];     ///申买价       
    int64_t     bidVol[5];       ///申买量       
    int64_t     auctioprice;    ///波动性中断参考价       
    int64_t     auctionQty;      ///波动性中断集合竞价虚拟匹配量       
};

#endif // __REMD_REMDAPISTRUCT_H__
