// @Author:   citicsf
// @Date:     2016/11/30
// @Brief:    

#ifndef __HSR__INI_PARSER__H__
#define __HSR__INI_PARSER__H__

#include <stdint.h>
#include <string>
#include <string.h>
#include <map>
#include <iostream>
#include <fstream>
#include <vector>

namespace remd {

// @Class:   IniParser
// @Author:  citicsf
// @Date:    2016/05/12
// @Brief:   
class IniParser {
    // max line buffer size is 1024
    static const uint32_t MAX_LINE_LENGTH = 1024*100;

    typedef std::string key_t;
    typedef std::string value_t;
    typedef std::string section_name_t;
    typedef std::map<key_t, value_t> key_value_t;
public:
    // @Class:   Section
    // @Author:  citicsf
    // @Date:    2016/07/04
    // @Brief:   
    class Section {
        friend class IniParser;
    public:
        // session name
        inline const char* Name() const { return section_name_.c_str(); }

        // get config value of key, if key can not be found
        // the return value is empty
        const char* GetValue(const key_t& key) const {
            auto cit = values_.find(key);
            return cit == values_.end() ? "" : cit->second.c_str();
        }

        // get all config values under this section
        key_value_t GetValues() const { return values_; }

        //
        void SetValue(const key_t& key, const value_t& value) {
            auto it = values_.find(key);
            if (it != values_.end()) { it->second = value; }
        }
        // does config have this key
        bool HasKey(const key_t& key) const {
            return values_.find(key) != values_.end();
        }
        // write into file
        bool Write(std::ofstream& file_stream) {
            if (section_name_.empty()) { return false; }
            file_stream << "[" << section_name_ << "]" << std::endl;
            auto end_pos = values_.end();
            for (auto it = values_.begin(); it != end_pos; ++it) {
                file_stream << it->first << " = " << it->second << "\n";
            }
            file_stream.flush();
            return true;
        }

    private:
        section_name_t section_name_;
        key_value_t values_;
    };

public:
    IniParser() : current_section_(nullptr) {}

    virtual ~IniParser() {
        for (auto section : sections_) {
            delete section;
        }
    }

    // read a ini file and parse it
    bool Read(const std::string& file_name) {
        std::ifstream file_stream(file_name);
        if (!file_stream.is_open()) { return false; }

        char line_buffer[MAX_LINE_LENGTH];
        while (!file_stream.eof()) {
            file_stream.getline(line_buffer, MAX_LINE_LENGTH);
            if (!parse_line(line_buffer)) {
                file_stream.close();
                return false;
            }
        }
        file_stream.close();
        return true;
    }

    // write all config into a file
    bool Write(const std::string& file_name) {
        std::ofstream file_stream(file_name);
        if (!file_stream.is_open()) { return false; }

        auto end_pos = sections_.end();
        for (auto it = sections_.begin(); it != end_pos; ++it) {
            if (!(*it)->Write(file_stream)) {
                file_stream.close();
                return false;
            }
        }
        file_stream.close();
        return true;
    }

    //    
    Section* FindSection(const section_name_t& section) const {
        for (auto ptr : sections_) {
            if (ptr->Name() == section) {
                return ptr;
            }
        }
        return nullptr;
    }

    inline const std::vector<Section*>& Sections() const { return sections_; }

    // get config value of key, if key can not be found
    // the return value is empty
    const char*
    GetValue(const section_name_t& section, const key_t& key) const {
        const Section* section_ptr = FindSection(section);
        return section_ptr == nullptr ? "" : section_ptr->GetValue(key);
    }

    void SetValue(const section_name_t& section,
                  const key_t& key,
                  const value_t& value) {
        Section* section_ptr = FindSection(section);
        if (section_ptr) {
            section_ptr->SetValue(key, value);
        }
    }

    // does config have this key
    bool HasKey(const section_name_t& section, const key_t& key) const {
        const Section* section_ptr = FindSection(section);
        return section_ptr ? section_ptr->HasKey(key) : false;
    }

protected:
    // create a new section and return it
    Section* new_section(const section_name_t& section) {
        Section* ptr = FindSection(section);
        if (ptr == nullptr) {
            ptr = new Section;
            ptr->section_name_ = section;
            sections_.push_back(ptr);
        }
        return ptr;
    }

    // 
    bool parse_line(const char* line_content) {
        if (line_content == NULL || is_line_end(line_content)) { return true; }

        // find the start pos of valid config, remove empty space
        const char* data_start = line_content;
        while (::isspace(*data_start)) data_start++;
        if (is_line_end(data_start) || *data_start == '=') { return true; }

        // is a new section?
        if (is_section(data_start)) { return true; }
        if (current_section_ == nullptr) { return false; }

        // find the end of valid config
        const char* data_end = data_start;
        while (!is_line_end(data_end)) { data_end++; }
        // remove empty space
        while (::isspace(*(--data_end)));

        // find the pos of '='
        const char* split_pos = strchr(data_start, '=');
        if (split_pos == NULL || split_pos >= data_end) { return true; }

        const char* key_end = split_pos - 1;
        // remove empty space
        while (::isspace(*key_end)) { key_end--; }

        const char* value_start = split_pos + 1;
        while (::isspace(*value_start)) { value_start++; }

        // save key-value
        current_section_->values_.insert(std::make_pair(
                std::string(data_start, key_end - data_start + 1),
                std::string(value_start, data_end - value_start + 1)));
        return true;
    }

    // is this character is an index of line ending?
    inline bool is_line_end(const char* p) const {
        return *p == '\0' || *p == '#' || *p == '\r' || *p == '\n';
    }

    bool is_section(const char* line_start) {
        if (*line_start != '[') { return false; }

        const char* line_end = (++line_start);
        while (*line_end != ']') { ++line_end; }

        if (is_line_end(line_end) || line_end == line_start) { return false; }

        section_name_t section_name(line_start, line_end - line_start);
        current_section_ = new_section(section_name);
        return true;
    }

protected:
    // key-value map
    std::vector<Section*> sections_;

    // current section
    Section* current_section_;
};

} // namespace hsr END


#endif // __HSR__INI_PARSER__H__
