﻿////////////////////////////////////////////////////////////////
// @File  net.h
// @Date  [2021/3/20/ yq]
// @Brief  
// 
////////////////////////////////////////////////////////////////


#ifndef net_h__
#define net_h__

#pragma once

#include <cstdio>
#include <stdlib.h>
#include <string>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <net/ethernet.h>
#include <fcntl.h>
#include <net/if.h>

#pragma pack(push)
#pragma pack(1)

struct TEthernetHdr {
    struct FrameHdr { 
        unsigned char DstMAC[6];    
        unsigned char SrcMAC[6];    
        uint16_t FrameType;         
    } framehdr;

    struct IPHdr {
        uint8_t  Ver_HLen;       
        uint8_t  TOS;            
        uint16_t TotalLen;       
        uint16_t ID;             
        uint16_t Flag_Segment;   
        uint8_t  TTL;            
        uint8_t  Protocol;       
        uint16_t Checksum;       
        uint32_t SrcIP;          
        uint32_t DstIP;          
    } iphdr;
};

//UDP数据报头(8=2*4)
struct TUDPHeader {
    uint16_t SrcPort;    //源端口
    uint16_t DstPort;    //目的端口
    uint16_t Len;        //长度
    uint16_t Checksum;   //校验
};
#pragma pack(pop)

struct TMulticastConfig {
    int socket;
    std::string localIp;
    std::string mcIp;
    int mcPort;
};

///231.77.13.68-->e74d0d44
static uint32_t Ipv4ToInteger(std::string sIp) {
    unsigned char ip[4] = { 0 };
    std::size_t pos, i = 0;
    while (true) {
        if ((pos = sIp.find_first_of('.')) == std::string::npos) {
            //最后一个无.分隔符
            ip[3] = stoi(sIp);
            break;
        }

        std::string str = sIp.substr(0, pos);
        sIp = sIp.substr(pos + 1);
        ip[i] = stoi(str);
        i++;
    }
    uint32_t ret(0);
    memcpy(&ret, ip, 4);
    return ret;
}

//组播接收初始化
static bool InitMulticastReceive(TMulticastConfig& config, bool nonBlockMode)
{
    printf("Init Multicast Receive: LocalIp %s, McIp %s, McPort %d\n",
        config.localIp.c_str(), config.mcIp.c_str(), config.mcPort);

    //Init Multicast receive
    config.socket = socket(AF_INET, SOCK_DGRAM, 0);
    if (config.socket < 0) {
        printf("socket error： %d - %s/n", config.socket, strerror(config.socket));
        return false;
    }

    //设置地址复用
    int enable = 1;
    setsockopt(config.socket, SOL_SOCKET, SO_REUSEADDR, &enable, sizeof(enable));

    //Set Local address
    struct sockaddr_in local_addr;
    memset(&local_addr, 0, sizeof(local_addr));
    local_addr.sin_family = AF_INET;
    local_addr.sin_addr.s_addr = htonl(INADDR_ANY);
    local_addr.sin_port = htons(config.mcPort);
    int ret = bind(config.socket, (struct sockaddr*) & local_addr, sizeof(local_addr));
    if (ret < 0) {
        printf("bind error: %d - %s/n", ret, strerror(ret));
        close(config.socket);
        return false;
    }

    //设置非阻塞模式
    if (nonBlockMode) {
        int flags = fcntl(config.socket, F_GETFL, 0);
        if (fcntl(config.socket, F_SETFL, flags | O_NONBLOCK) < 0) {
            close(config.socket);
            printf("Set NONBLOCK failed\n");
            return false;
        }
    }

    struct ip_mreq mreq; // 多播地址结构体
    mreq.imr_multiaddr.s_addr = inet_addr(config.mcIp.data());
    mreq.imr_interface.s_addr = inet_addr(config.localIp.data());

    ret = setsockopt(config.socket, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq,
        sizeof(mreq));
    if (ret < 0) {
        printf("add membership error： %d - %s/n", ret, strerror(ret));
        close(config.socket);
        return false;
    }

    printf("Nic(%s)Init Multicast Receive %s:%d.\n", config.localIp.data(), config.mcIp.data(), config.mcPort);
    return true;
}

static bool ExitMulticastReceive(TMulticastConfig& config)
{
    struct ip_mreq mreq; // 多播地址结构体
    mreq.imr_multiaddr.s_addr = inet_addr(config.mcIp.data());
    mreq.imr_interface.s_addr = inet_addr(config.localIp.data());
    int ret = setsockopt(config.socket, IPPROTO_IP, IP_DROP_MEMBERSHIP, &mreq, sizeof(mreq));
    if (ret < 0) {
        printf("Drop membership error： %d - %s/n", ret, strerror(ret));
        return false;
    }
    close(config.socket);
    printf("Nic(%s)Exit Multicast Receive %s:%d.\n", config.localIp.data(), config.mcIp.data(), config.mcPort);
    return true;
}

static bool GetNicbyIp(const std::string localAddr, std::string& Nic)
{
    int fdSock = 0;
    struct ifconf ifMyConf;
    struct ifreq ifMyReq;
    char szBuf[2048] = { 0 };

    ifMyConf.ifc_len = 2048;
    ifMyConf.ifc_buf = szBuf;

    if ((fdSock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
        printf("socket error\n");
        return false;
    }

    if (ioctl(fdSock, SIOCGIFCONF, &ifMyConf))
    {
        close(fdSock);
        printf("ioctl error \n");
        return false;
    }

    ifreq* it = ifMyConf.ifc_req;
    const struct ifreq* const end = it + (ifMyConf.ifc_len / sizeof(struct ifreq));

    for (; it != end; ++it) {
        strcpy(ifMyReq.ifr_name, it->ifr_name);
        if (ioctl(fdSock, SIOCGIFADDR, &ifMyReq) == 0) {
            std::string strAddr;
            struct sockaddr_in* sin;
            sin = (struct sockaddr_in*) & ifMyReq.ifr_addr;

            strAddr = (const char*)inet_ntoa(sin->sin_addr);
            if (strcmp(strAddr.c_str(), localAddr.c_str()) == 0)
            {
                Nic = ifMyReq.ifr_name;
                break;
            }
        }
        else {
            printf("get mac info error \n");
        }
    }
    close(fdSock);
    if (Nic.size() != 0) {
        return true;
    }
    else {
        printf("get nic info error \n");
    }
    return false;
}

static bool InitMiscModeReceive(TMulticastConfig& config)
{
    //IP对应网卡名称
    std::string localNicName;
    if (!GetNicbyIp(config.localIp, localNicName)) {
        printf("Nic(%s) Init MiscMode Receive Get NicName Failed %s:%d.\n",
            config.localIp.data(), config.mcIp.data(), config.mcPort);
        return false;
    }
    else {
        printf("Set listen to Nic(%s)\n", localNicName.data());
    }

    if ((config.socket = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0)
    {
        printf("SetMiscMode() create socket failed\n");
        return false;
    }

#if 1
    //设置非阻塞模式
    int flags = fcntl(config.socket, F_GETFL, 0);
    if (fcntl(config.socket, F_SETFL, flags | O_NONBLOCK) < 0)
    {
        printf("Set NONBLOCK failed\n");
        return false;
    }
#endif

    struct ifreq ethreq;
    memset(&ethreq, 0, sizeof(ethreq));
    strncpy(ethreq.ifr_name, localNicName.data(), IFNAMSIZ);
    if (ioctl(config.socket, SIOCGIFFLAGS, &ethreq) < 0)
    {
        printf("SetMiscMode() Get interface flag failed\n");
        close(config.socket);
        return false;
    }

    if (ethreq.ifr_flags & IFF_RUNNING) {
        printf("%s link up\n", localNicName.data());
    }
    else {
        printf("%s link down\n", localNicName.data());
    }

    /* add the misc mode */
    ethreq.ifr_flags |= IFF_PROMISC;
    if (ioctl(config.socket, SIOCSIFFLAGS, &ethreq) < 0)
    {
        printf("SetMiscMode() Set interface flag failed\n");
        close(config.socket);
        return false;
    }

    printf("Nic(%s)Init MiscMode Receive %s:%d.\n", config.localIp.data(), config.mcIp.data(), config.mcPort);
    return true;
}


#endif // net_h__
